#!@l_prefix@/lib/openpkg/bash
##
##  register -- OpenPKG Registry Command-Line Client
##  Copyright (c) 2000-2005 OpenPKG Foundation e.V. <http://openpkg.net/>
##  Copyright (c) 2000-2005 Ralf S. Engelschall <http://engelschall.com/>
##
##  Permission to use, copy, modify, and distribute this software for
##  any purpose with or without fee is hereby granted, provided that
##  the above copyright notice and this permission notice appear in all
##  copies.
##
##  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
##  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
##  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
##  IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS AND THEIR
##  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
##  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
##  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
##  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
##  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
##  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
##  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
##  SUCH DAMAGE.
##

##
##  configuration
##

#   program name, version and date
progname="register"
progvers="0.7.2"
progdate="09-Aug-2006"

#   determine path to OpenPKG instance
PREFIX="@l_prefix@"
if [ ".${OPENPKG_PREFIX}" != . ]; then
    PREFIX="${OPENPKG_PREFIX}"
fi

#   determine rpm
rpm="$PREFIX/libexec/openpkg/rpm"
[ -x "$PREFIX/lib/openpkg/rpm" ] && rpm="$PREFIX/lib/openpkg/rpm"

#   http post
curl="$PREFIX/lib/openpkg/curl"
useragent="openpkg-$progname/$progvers"

error()
{
    echo "$progname:ERROR: $@" 1>&2
    exit 1
}

[ ".$PREFIX" = . ] && error "empty PREFIX not allowed"

##
##  read configuration unless already done during option parsing
##
sanitycheck()
{
    if [ ".$1" != . -a -r "$1" ]; then
        cat <"$1" | awk '
            BEGIN { rc=0 }
            !/^([A-Z][A-Z0-9_]+[A-Z0-9]="[^"]*")? *(#.*)?$/ { rc=1 }
            END { exit rc }' && return 0
    fi
    return 1
}

readconf()
{
    [ ".$REGISTRY_CONF" = . ] && return
    if [ -r "$REGISTRY_CONF" ]; then
        sanitycheck "$REGISTRY_CONF" && . "$REGISTRY_CONF"
    fi
}

readuuid()
{
    [ ".$REGISTRY_UUID" = . ] && return
    if [ -r "$REGISTRY_UUID" ]; then
        sanitycheck "$REGISTRY_UUID" && . "$REGISTRY_UUID"
    fi
}

readutil()
{
    [ ".$REGISTRY_UTIL" = . ] && return
    if [ -r "$REGISTRY_UTIL" ]; then
        sanitycheck "$REGISTRY_UTIL" && . "$REGISTRY_UTIL"
    fi
}

##
##  command line option parsing
##

#   default parameters

#   primary operations
op="automatic"

#   standard options
REGISTRY_MODE=""
REGISTRY_ARGS="http://registry.openpkg.org/register"
REGISTRY_USER=""
REGISTRY_LINK=""
REGISTRY_DESC="openpkg://${HOSTNAME}${PREFIX}"

#   advanced options
#   we are called from rpm URL rewrite wrapper,
#   so defer PLAT and OREL lookup to avoid rpm loop
REGISTRY_PLAT=""
REGISTRY_OREL=""
REGISTRY_UUID="$PREFIX/etc/openpkg/uuid"
REGISTRY_CONF="$PREFIX/etc/openpkg/register.conf"
REGISTRY_PREP="$PREFIX/etc/openpkg/register.prep"
REGISTRY_TRAN="$PREFIX/etc/openpkg/register.tran"
REGISTRY_UTIL="$PREFIX/etc/openpkg/register.util"

#   amount of data being posted
REGISTRY_DATA="request,package,provides"

#   read baseline config early and allow options to override contents
readconf

#   debug options
verbose="no"
help="no"

#   iterate over argument line
declare -a a
declare -i i=0
while [ $# -gt 0 ]; do
    opt=$1
    case $opt in
        -*=*) arg=`echo "${opt}" | sed 's/^[-_a-zA-Z0-9]*=//'`
              opt=`echo "${opt}" | sed 's/=.*//'`
              ;;
           *) arg='' ;;
    esac
    case $opt in
        #   primary operations
        -P|--preparation ) op="preparation" ;;
        -T|--transaction ) op="transaction" ;;
        -U|--utilization ) op="utilization" ;;
        -C|--convenience ) op="convenience" ;;
        -I|--interaction ) op="interaction" ;;

        #   additional features
        -S|--printstatus ) op="printstatus" ;;
        -R|--rewriteurls ) op="rewriteurls" ;;

        #   standard options
        -m|--mode )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_MODE="$arg"
            ;;
        -a|--args )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_ARGS="$arg"
            ;;
        -u|--user )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_USER="$arg"
            ;;
        -l|--link )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_LINK="$arg"
            ;;
        -d|--desc )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_DESC="$arg"
            ;;

        #   advanced options
        --plat )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_PLAT="$arg"
            ;;
        --orel )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_VERS="$arg"
            ;;
        --uuid )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_UUID="$arg"
            ;;
        --conf )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_CONF="$arg"; readconf
            ;;
        --prep )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_PREP="$arg"
            ;;
        --tran )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_TRAN="$arg"
            ;;
        --util )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_UTIL="$arg"
            ;;
        --data )
            if [ ".$arg" = . ]; then shift; arg="$1"; fi
            if [ ".$arg" = . ]; then help="Missing argument to \"$opt\""; break; fi
            REGISTRY_DATA="$arg"
            ;;

        #   debug options
        -v|--verbose ) verbose="yes" ;;
        -h|--help    ) help="Usage"  ;;
        -*           ) help="Invalid option \`$opt'"; break ;;

        #   optional arguments
        * ) a[$i]="$opt"
            i=i+1
            ;;
    esac
    shift
done
set -- "${a[@]}"
unset i a

#   advanced options, lookup PLAT and OREL default for all modes
#   but those called from rpm URL rewrite mapper
if [ $op != "printstatus" -a $op != "rewriteurls" ]; then
    [ ".$REGISTRY_PLAT" = . ] && REGISTRY_PLAT="`$rpm --eval '%{l_platform -p}'`"
    [ ".$REGISTRY_OREL" = . ] && REGISTRY_OREL="`$rpm --eval '%{l_openpkg_release}'`"
fi

#   display error or usage message
if [ ".$help" != .no ]; then
    if [ ".$help" != ".Usage" ]; then
        echo "$progname:ERROR: $help" 1>&2
    fi
    echo "Usage: $progname -m|--mode=fake|post|wipe [-a|--args=<args>]"
    echo "       -u|--user=<user> [-d|--desc=<text>]"
    echo "       [--plat=<text>] [--orel=<text>] [--uuid=<file>]"
    echo "       [--conf=<file>] [--prep=<file>] [--tran=<file>] [--util=<file>] [--data=<tag>[,<tag>...]]"
    echo "       [-P|--preparation] [-T|--transaction] [-U|--utilization]"
    echo "       [-C|--convenience] [-I|--interaction]"
    echo "       [-v|--verbose] [-h|--help]"
    echo "Usage: $progname -S|--printstatus"
    echo "Usage: $progname -R|--rewriteurls [[url]...]]"
    if [ ".$help" != ".Usage" ]; then
        exit 1
    else
        exit 0
    fi
fi

##
##  Primary operation Preparation
##
preparation()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        [ ".$REGISTRY_PREP" != . -f $REGISTRY_PREP ] && cat </dev/null >$REGISTRY_PREP
        return
    fi

    [ ".$REGISTRY_USER" = . ] && error "missing information REGISTRY_USER"
    [ ".$REGISTRY_DESC" = . ] && error "missing information REGISTRY_DESC"
    [ ".$REGISTRY_PLAT" = . ] && error "missing information REGISTRY_PLAT"
    [ ".$REGISTRY_OREL" = . ] && error "missing information REGISTRY_OREL"
    [ ".$REGISTRY_UUID" = . ] && error "missing information REGISTRY_UUID"
    [ ".$REGISTRY_PREP" = . ] && error "missing information REGISTRY_PREP"

    if [ -r "$REGISTRY_UUID" ]; then
        sanitycheck "$REGISTRY_UUID" && . "$REGISTRY_UUID"
    fi
    [ ".$UUID_REGISTRY" = . ] && error "missing information UUID_REGISTRY"
    [ ".$UUID_INSTANCE" = . ] && error "missing information UUID_INSTANCE"
    [ ".$UUID_PLATFORM" = . ] && error "missing information UUID_PLATFORM"

    #   amount of data being posted
    Q='"'; I="    "; N=$'\n'; D=""

    echo "$REGISTRY_DATA" | grep request >/dev/null
    if [ $? = 0 ]; then
        D="${D}<?xml version=${Q}1.0${Q} encoding=${Q}iso-8859-1${Q} standalone=${Q}no${Q}?>${N}"
        D="${D}<!DOCTYPE registry${N}"
        D="${D}  PUBLIC ${Q}-//OpenPKG//DTD OpenPKG Registry 0.0.1//EN${Q}${N}"
        D="${D}  ${Q}http://registry.openpkg.org/registry.dtd${Q} []>${N}"
        D="${D}<registry>${N}"
        D="${D}${I}<request id=${Q}$UUID_REGISTRY${Q}${N}"
        D="${D}${I}${I}registry_user=${Q}$REGISTRY_USER${Q}${N}"
        [ ".$REGISTRY_LINK" != . ] && \
        D="${D}${I}${I}registry_link=${Q}$REGISTRY_LINK${Q}${N}" # optional
        D="${D}${I}${I}registry_desc=${Q}$REGISTRY_DESC${Q}${N}"
        D="${D}${I}${I}registry_plat=${Q}$REGISTRY_PLAT${Q}${N}"
        D="${D}${I}${I}registry_orel=${Q}$REGISTRY_OREL${Q}${N}"
        D="${D}${I}${I}uuid_registry=${Q}$UUID_REGISTRY${Q}${N}"
        D="${D}${I}${I}uuid_instance=${Q}$UUID_INSTANCE${Q}${N}"
        D="${D}${I}${I}uuid_platform=${Q}$UUID_PLATFORM${Q}${N}"

        echo "$REGISTRY_DATA" | grep package >/dev/null
        if [ $? = 0 ]; then
            D="${D}${I}>${N}"
            F=""
            F="${F}${I}${I}<package id=${Q}%{PKGID}${Q} name=${Q}%{NAME}${Q} version=${Q}%{VERSION}${Q} release=${Q}%{RELEASE}${Q}"
            echo "$REGISTRY_DATA" | grep provides >/dev/null
            if [ $? = 0 ]; then
                F="${F}>\n"
                F="${F}[${I}${I}${I}<provides name=${Q}%{PROVIDENAME}${Q} flag=${Q}%{PROVIDEFLAGS:depflags}${Q} version=${Q}%{PROVIDEVERSION}${Q}/>\n]"
                F="${F}${I}${I}</package>\n"
            else
                F="${F}/>\n"
            fi
            D="${D}`$rpm --qf \"${F}\" -qa | sed '/<package id="(none)"/,/<\/package>/d'`${N}"
            D="${D}    </request>${N}"
        else
            D="${D}${I}/>${N}"
        fi
        D="${D}</registry>${N}"
    fi
    echo "$D" | tee $REGISTRY_PREP
}

##
##  Primary operation Transaction
##
transaction()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        [ ".$REGISTRY_TRAN" != . -f $REGISTRY_TRAN ] && cat </dev/null >$REGISTRY_TRAN
        return
    fi

    [ ".$REGISTRY_MODE" = . ] && error "missing information REGISTRY_MODE"
    [ ".$REGISTRY_ARGS" = . ] && error "missing information REGISTRY_ARGS"
    [ ".$REGISTRY_TRAN" = . ] && error "missing information REGISTRY_TRAN"

    if [ ".$REGISTRY_MODE" = .fake ]; then
        request=`awk '
            BEGIN { registry=0 }
            /<registry>/ { registry=1 }
            /<request / { if (registry) { print $0 } }
            /<\/registry>/ { registry=0 }' | \
            sed -e 's;^.*<request id=";;' -e 's;"$;;'`
        if [ ".$request" = . ]; then error "no request seen on stdin"; fi
        (
            echo "<?xml version=\"1.0\" encoding=\"iso-8859-1\" standalone=\"no\"?>"
            echo "<!DOCTYPE registry"
            echo "  PUBLIC \"-//OpenPKG//DTD OpenPKG Registry 0.0.1//EN\""
            echo "  \"http://registry.openpkg.org/registry.dtd\" []>"
            echo "<registry>"
            echo "    <response id=\"$request\" done=\"yes\">fake</response>"
            echo "</registry>"
        ) | tee $REGISTRY_TRAN
    fi

    if [ ".$REGISTRY_MODE" = .post ]; then
        O=""
        if [ $verbose = no ]; then O="$O --silent"; fi
        $curl \
            $O \
            --user-agent "$useragent" \
            --form 'page=dropxml' \
            --form 'data=<-' \
            --form 'submit=DROPXML' \
            $REGISTRY_ARGS \
        | tee $REGISTRY_TRAN
    fi
}

##
##  Primary operation Utilization
##
utilization()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        [ ".$REGISTRY_UTIL" != . -f $REGISTRY_UTIL ] && cat </dev/null >$REGISTRY_UTIL
        return
    fi

    [ ".$REGISTRY_UTIL" = . ] && error "missing information REGISTRY_UTIL"

    response=`awk '
        BEGIN { registry=0 }
        /<registry>/ { registry=1 }
        /<response / { if (registry) { print $0 } }
        /<\/registry>/ { registry=0 }' | \
        sed -e 's;^.*<response id=";;' -e 's;" done="; ;' -e 's;">; ;' -e 's;</response>.*$;;'`
    if [ ".$response" = . ]; then error "no response seen on stdin"; fi

    REGISTRY_UUID=`echo $response | awk '/./ { print $1 }'`
    REGISTRY_DONE=`echo $response | awk '/./ { print $2 }'`
    REGISTRY_RESP=`echo $response | sed -e 's;^[^ ][^ ]* [^ ][^ ]* ;;'`

    [ ".$REGISTRY_UUID" = . ] && error "missing information REGISTRY_UUID"
    [ ".$REGISTRY_DONE" = . ] && error "missing information REGISTRY_DONE"
    [ ".$REGISTRY_RESP" = . ] && error "missing information REGISTRY_RESP"

    (
        echo "REGISTRY_UUID=\"$REGISTRY_UUID\""
        echo "REGISTRY_DONE=\"$REGISTRY_DONE\""
        echo "REGISTRY_RESP=\"$REGISTRY_RESP\""
    ) | tee $REGISTRY_UTIL
}

##
##  Primary operation Convenience
##
convenience()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        utilization
        transaction
        preparation
        return
    fi

    preparation | transaction | utilization
}

##
##  Primary operation Interaction
##
interaction()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        [ ".$REGISTRY_CONF" != . -f $REGISTRY_CONF ] && cat </dev/null >$REGISTRY_CONF
        return
    fi

    echo "OpenPKG Registry Client ($useragent)"
    echo ""
    echo "Interactive step-by-step registration procedure."
    echo "You can abort at any time by just pressing CTRL-C."
    echo ""
    echo "Registration is a three step process:"
    echo "1. Preparation = create a configuration file and locally prepare a registration request"
    echo "2. Transaction = run a transaction to submit the request to the registry"
    echo "3. Utilization = memorize the response to activate local URL rewriting"
    echo ""
    echo "A (*) marks an optional field where the empty string \"\" is allowed."
    echo "Just press RETURN to continue and/or keep a default setting."

    echo ""
    echo "==== Step 1/3: Preparation ===="
    echo "Attempting to write to the configuration file \"$REGISTRY_CONF\""
    echo "#`TZ= date '+%Y-%m-%d %H:%M:%S'`" | tee -a "$REGISTRY_CONF" 2>/dev/null \
        || error "Configuration file not writable. Get more permissions and try again"

    read -p "[REGISTRY_USER=\"$REGISTRY_USER\"] username? ";          [ ".$REPLY" != . ] && REGISTRY_USER="$REPLY"
    read -p "[REGISTRY_DESC=\"$REGISTRY_DESC\"] description (*)? ";   [ ".$REPLY" != . ] && REGISTRY_DESC="$REPLY"
    read -p "[REGISTRY_PLAT=\"$REGISTRY_PLAT\"] platform (*)? ";      [ ".$REPLY" != . ] && REGISTRY_DESC="$REPLY"
    read -p "[REGISTRY_OREL=\"$REGISTRY_OREL\"] release (*)? ";       [ ".$REPLY" != . ] && REGISTRY_DESC="$REPLY"
    read -p "[REGISTRY_UUID=\"$REGISTRY_UUID\"] uuid file to read? "; [ ".$REPLY" != . ] && REGISTRY_UUID="$REPLY"
    read -p "[REGISTRY_DATA=\"$REGISTRY_DATA\"] data to prepare? ";   [ ".$REPLY" != . ] && REGISTRY_DATA="$REPLY"
    read -p "[REGISTRY_PREP=\"$REGISTRY_PREP\"] prep dump file? ";    [ ".$REPLY" != . ] && REGISTRY_PREP="$REPLY"

    [ ".$REGISTRY_USER" = . ] && error "missing information REGISTRY_USER"
    [ ".$REGISTRY_DESC" = . ] && error "missing information REGISTRY_DESC"
    [ ".$REGISTRY_PLAT" = . ] && error "missing information REGISTRY_PLAT"
    [ ".$REGISTRY_OREL" = . ] && error "missing information REGISTRY_OREL"
    [ ".$REGISTRY_UUID" = . ] && error "missing information REGISTRY_UUID"
    [ ".$REGISTRY_DATA" = . ] && error "missing information REGISTRY_DATA"
    [ ".$REGISTRY_PREP" = . ] && error "missing information REGISTRY_PREP"

    if [ -r "$REGISTRY_UUID" ]; then
        sanitycheck "$REGISTRY_UUID" && . "$REGISTRY_UUID"
    fi
    [ ".$UUID_REGISTRY" = . ] && error "missing information UUID_REGISTRY"
    [ ".$UUID_INSTANCE" = . ] && error "missing information UUID_INSTANCE"
    [ ".$UUID_PLATFORM" = . ] && error "missing information UUID_PLATFORM"

    echo "Appending to configuration file \"$REGISTRY_CONF\""
    (
        echo "REGISTRY_USER=\"$REGISTRY_USER\""
        echo "REGISTRY_LINK=\"$REGISTRY_LINK\""
        echo "REGISTRY_DESC=\"$REGISTRY_DESC\""
        echo "REGISTRY_PLAT=\"$REGISTRY_PLAT\""
        echo "REGISTRY_OREL=\"$REGISTRY_OREL\""
        echo "REGISTRY_UUID=\"$REGISTRY_UUID\""
        echo "REGISTRY_PREP=\"$REGISTRY_PREP\""
    ) | tee -a "$REGISTRY_CONF"

    preparation

    echo ""
    echo "==== Step 2/3: Transaction ===="
    read -p "[REGISTRY_MODE=\"$REGISTRY_MODE\"] mode (post or fake)? "; [ ".$REPLY" != . ] && REGISTRY_MODE="$REPLY"
    read -p "[REGISTRY_ARGS=\"$REGISTRY_ARGS\"] args? ";                [ ".$REPLY" != . ] && REGISTRY_ARGS="$REPLY"
    read -p "[REGISTRY_TRAN=\"$REGISTRY_TRAN\"] tran dump file? ";      [ ".$REPLY" != . ] && REGISTRY_TRAN="$REPLY"

    [ ".$REGISTRY_MODE" = . ] && error "missing information REGISTRY_MODE"
    [ ".$REGISTRY_ARGS" = . ] && error "missing information REGISTRY_ARGS"
    [ ".$REGISTRY_TRAN" = . ] && error "missing information REGISTRY_TRAN"

    echo "Appending to configuration file \"$REGISTRY_CONF\""
    (
        echo "REGISTRY_MODE=\"$REGISTRY_MODE\""
        echo "REGISTRY_ARGS=\"$REGISTRY_ARGS\""
        echo "REGISTRY_TRAN=\"$REGISTRY_TRAN\""
    ) | tee -a "$REGISTRY_CONF"

    transaction <$REGISTRY_PREP

    echo ""
    echo "==== Step 3/3: Utilization ===="
    read -p "[REGISTRY_UTIL=\"$REGISTRY_UTIL\"] util dump file? ";      [ ".$REPLY" != . ] && REGISTRY_UTIL="$REPLY"

    [ ".$REGISTRY_UTIL" = . ] && error "missing information REGISTRY_UTIL"

    echo "Appending to configuration file \"$REGISTRY_CONF\""
    (
        echo "REGISTRY_UTIL=\"$REGISTRY_UTIL\""
    ) | tee -a "$REGISTRY_CONF"

    utilization <$REGISTRY_TRAN
}

##
##  Additional feature printstatus
##
printstatus()
{
    rc=0
    readconf || error "problem reading conf"
    readuuid || error "problem reading uuid"
    readutil || error "problem reading util"
    if [ ".$REGISTRY_DONE" = .yes ]; then
        cat "$REGISTRY_UTIL"
        return 0
    fi
    return 1
}

##
##  Additional feature rewriteurls
##
rewriteurls()
{
    readconf && readuuid && readutil || return $?
    if [ ".$REGISTRY_DONE" = .yes ]; then
        #   URL rewriting
        while [ ".$1" != . ]; do
            printf '%s\n' "$1" | \
            sed -e "s;^\([^:/]*\)://\([^:/]*\.openpkg\.com\)/;\1://$UUID_REGISTRY:$UUID_INSTANCE.$UUID_PLATFORM@\2/;" \
                -e "s;^\([^:/]*\)://\([^:/]*\.openpkg\.org\)/;\1://$UUID_REGISTRY:$UUID_INSTANCE.$UUID_PLATFORM@\2/;" \
                -e "s;^\([^:/]*\)://\([^:/]*\.openpkg\.net\)/;\1://$UUID_REGISTRY:$UUID_INSTANCE.$UUID_PLATFORM@\2/;"
            shift
        done
        return 0
    else
        #   stealth mode
        return 1
    fi
}

##
##  automatically pick interaction or convenience
##
automatic()
{
    if [ ".$REGISTRY_MODE" = .wipe ]; then
        convenience
        interaction
        return
    fi

    missing=""
    [ ".$REGISTRY_USER" = . ] && missing="$missing REGISTRY_USER"
    [ ".$REGISTRY_DESC" = . ] && missing="$missing REGISTRY_DESC"
    [ ".$REGISTRY_PLAT" = . ] && missing="$missing REGISTRY_PLAT"
    [ ".$REGISTRY_OREL" = . ] && missing="$missing REGISTRY_OREL"
    [ ".$REGISTRY_UUID" = . ] && missing="$missing REGISTRY_UUID"
    [ ".$REGISTRY_PREP" = . ] && missing="$missing REGISTRY_PREP"
    [ ".$REGISTRY_MODE" = . ] && missing="$missing REGISTRY_MODE"
    [ ".$REGISTRY_ARGS" = . ] && missing="$missing REGISTRY_ARGS"
    [ ".$REGISTRY_TRAN" = . ] && missing="$missing REGISTRY_TRAN"
    [ ".$REGISTRY_UTIL" = . ] && missing="$missing REGISTRY_UTIL"

    if [ ".$missing" = . ]; then
        convenience
    else
        echo "missing $missing"
        interaction
    fi
}

##
##  primary operation switch
##
eval $op "\"$@\""
exit $?

##
##  MANUAL PAGE
##

=pod

=head1 NAME

B<openpkg register> - OpenPKG Registry Command-Line Client

=head1 SYNOPSIS

B<register> -m|--mode=fake|post|wipe [I<-a|--args=<args>>]
         -u|--user=<user|token> [-u|--link=<token>] [I<-d|--desc=<text>>]
          [I<--plat=<text>>] [I<--orel=<text>>] [I<--uuid=<file>>]
          [I<--conf=<file>>] [I<--prep=<file>>] [I<--tran=<file>>]
          [I<--util=<file>>] [--data=<tag>[,<tag>...]]
          [I<-P|--preparation>] [I<-T|--transaction>]
          [I<-U|--utilization>] [I<-C|--convenience>]
          [I<-I|--interaction>]
          [I<-v|--verbose>] [I<-h|--help>]

B<register> -S|--printstatus

B<register> -R|--rewriteurls [I<url> ...]

=head1 DESCRIPTION

B<openpkg register> is the Command-Line Client the OpenPKG Registry. It is
used by administrators to register an instance with the OpenPKG Registry for
later association.

Automatic association is intentionally not possible and login to the web form
on http://registry.openpkg.org/ is required.

After association, B<openpkg register> can be used for repetitive
reregistrations which update the heartbeat of the instance on the Registry
server, avoiding premature depature from the database. It is assumed that
every instance will be reregistered every quarter of the year. Dormant
instances might be discarded from the Registry, revoking their access to
additional resources.

=head1 PRIMARY OPERATIONS

The following primary operations are available:

=over 4

=item B<-h>, B<--help>

Display brief usage message.

=item B<-v>, B<--verbose>

Display progress information during data posting.

=item B<-P>, B<--preparation>

Execute the primary operation "preparation".
Creates registry data for one request and dumps it stdout
in XML format. That request can be filtered and piped into
the transaction phase or it can be copied and pasted into the
XMLdump CGI facility manually.
A copy of the output of the last run is also saved to the C<${REGISTRY_PREP}> file.

=item B<-T>, B<--transaction>

Execute the primary operation "transaction".
Reads registry data with one request from stdin, executes
the transaction and writes registry data with one or more
responses to stdout in XML format.
Depending on the mode of operation, the transaction might be an actual
transport to the XMLdump CGI facility or a fake activity which complements the
manual preparation.
A copy of the output of the last run is also saved to the C<${REGISTRY_TRAN}> file.

=item B<-U>, B<--utilization>

Execute the primary operation "utilization".
Reads registry data with one response from stdin and updates the local
registry information.
Depending on the mode of operation, the utilization might be an actual
processing of a transaction response or a fake activity which complements the
manual transaction.
Anyway, this step finalizes the registration process and makes the instance
assume it has been properly registered. This status can be printed. URL
rewriting is activated.
A copy of the output of the last run is also saved to the C<${REGISTRY_UTIL}> file.

=item B<-C>, B<--convenience>

Execute the primary operation "convenience".
This executes the three primary operations "preparation", "transaction" and
"utilization" in that order and pipes data through this chain.

=item B<-I>, B<--interaction>

Execute the primary operation "interaction".
Like "convenience" but user is interactively asked for information. This is
the easiest way to do registration but it is not meat to be automated.

=item B<-S>, B<--printstatus>

If the instance has been registered, information about the registration is printed in a
format suitable for shell evaluation and return code is true.
Otherwise nothing is printed and return code is false.

=item B<-R>, B<--rewriteurls> [I<url> ...]

If the instance has been registered, the given URLs are rewritten to prepend
user:pass information before hostnames below the openpkg.(org|net|com)
domains.  Note the username is is UUID_REGISTRY and the password is the
concatenation of UUID_INSTANCE and UUID_PLATFORM from the
PREFIX/etc/openpkg/uuid file. Both informations are not meant to be used for
traditional authentication, they are merly statistical information.
Otherwise the URLs are returned verbatim.

=back

=head1 STANDARD OPTIONS

Standard options are typically used to automate registration (not
association).

=over 4

=item B<-m>, B<--mode> fake|post|wipe

Overrides C<${REGISTRY_MODE}> variable in C<${REGISTRY_CONF}> file.
Has no default and is a manadatory setting.

In B<post> mode, transactions are carried out using real network connectivity.
The "preparation" step creates a XMLdump and writes it into the
C<${REGISTRY_PREP}> file. The "transaction" step posts it to the DropXML form
using a HTTP request. The response coming back from Registration server is
also in XML format and is saved to  C<${REGISTRY_TRAN}> file.  The
"utilization" step processes this response and writes the final results to the
C<${REGISTRY_UTIL}> file.

In B<fake> mode, no network connectivity takes place.
The "prepararation" step creates a XMLdump and writes it into the
C<${REGISTRY_PREP}> file. The "transaction" step is a fake only. It assumes a
successful response from the Registration server and saves it to
C<${REGISTRY_TRAN}> file.  The "utilization" step processes this response and
writes the final results to the C<${REGISTRY_UTIL}> file.
I<Note>: fake mode is meant as a way to register instances which cannot or must
not post data directly to the Registration server.

In B<wipe> mode, the registration is wiped out locally. The status is reset
and URL rewriting is disabled. 
I<Note>: the registration server is not contacted, the instace must be removed
manually using the web interface.
I<Note>: wiping registration is highly recommended as a precursor action of
cloning activities.

=item B<-a>, B<--args> "arg [arg ...]"

Overrides C<${REGISTRY_ARGS}> variable in C<${REGISTRY_CONF}> file.
Complements the mode and is specific to it.
Defaults to "http://registry.openpkg.org/register" which is the official registry of the OpenPKG Project.
This default is useful for "post" mode.

=item B<-u>, B<--user> I<user|token>

Overrides C<${REGISTRY_USER}> variable in C<${REGISTRY_CONF}> file.
Indicates the registry user which will find this registration in his arrival queue.
The user can also specified as a token which is enabled for use as "user".
If the token is also enabled for "assoc" the registered instance is immediately associated
to the user with no need for the user to visit the web interface.
This information in submitted via the "registry_user" attribute of the XML request.
Has no default and is a manadatory setting.

=item B<-l>, B<--link> I<token>

Overrides C<${REGISTRY_LINK}> variable in C<${REGISTRY_CONF}> file.
Indicates the registry user which will find this registration linked to his token.
The link must be specified as a token which is enabled for use as "link".
If a user token is used and the link token is also enabled for "assoc" then the registered instance
is immediately associated to the user with no need for the user to visit the web interface.
This information in submitted via the "registry_link" attribute of the XML request.
Has no default and is an optional setting.
Be aware that linked users see the same information about an instance as the associated user
can find on his "association" page but they cannot alter or delete information.

=item B<-d>, B<--desc> I<description>

Overrides C<${REGISTRY_DESC}> variable in C<${REGISTRY_CONF}> file.
Indicates a human readable description of the instance.
This information in submitted via the "registry_desc" attribute of the XML request.
It appears in the "description" column on the "association" page of the web form and can be edited on the server side.
Defaults to "openpkg://${HOSTNAME}${PREFIX}"

=back

=head1 ADVANCED OPTIONS

Advanced options can be enganged to tailor and fully automate a registration
(not association).

=over 4

=item B<--plat>

Overrides C<${REGISTRY_PLAT}> variable in C<${REGISTRY_CONF}> file.
Indicates the platform. Concatenated information from CPU architecture (arch) and Operating System (os).
This information in submitted via the "registry_plat" attribute of the XML request.
Defaults to "%{l_platform -p}", e.g. "ix86-freebsd6.1"

=item B<--orel>

Overrides C<${REGISTRY_VERS}> variable in C<${REGISTRY_CONF}> file.
Indicates the OpenPKG release.
This information in submitted via the "registry_orel" attribute of the XML request.
Defaults to "%{l_openpkg_release}", e.g. "OpenPKG-CURRENT", "OpenPKG-2-STABLE", "OpenPKG-2.5",

=item B<--uuid>

Overrides C<${REGISTRY_UUID}> variable in C<${REGISTRY_CONF}> file.
Indicates the UUID file of the instance.
This information in submitted via the "uuid_registry", "uuid_instance" and "uuid_platform" attributes of the XML request.
Defaults to F<${PREFIX}/etc/openpkg/uuid>

=item B<--conf>

Overrides C<${REGISTRY_CONF}> variable from previous C<${REGISTRY_CONF}> file. Processing
of the current configuration file is aborted immediately with all variables
read so far kept. THe new configuration file is read in immediately and
processing continues there. This works similar to an include but is more
primitive as it does not allow nesting, only chaining.
Defaults to F<${PREFIX}/etc/openpkg/register.conf>

=item B<--prep>

Overrides C<${REGISTRY_PREP}> variable in C<${REGISTRY_CONF}> file.
File to save a copy of the output from the "preparation" step.
Defaults to F<${PREFIX}/etc/openpkg/register.prep>

=item B<--tran>

Overrides C<${REGISTRY_TRAN}> variable in C<${REGISTRY_CONF}> file.
File to save a copy of the output from the "transaction" step.
Defaults to F<${PREFIX}/etc/openpkg/register.tran>

=item B<--util>

Overrides C<${REGISTRY_UTIL}> variable in C<${REGISTRY_CONF}> file.
File to save a copy of the output from the "utilization" step.
Defaults to F<${PREFIX}/etc/openpkg/register.util>

=item B<--data>

Overrides C<${REGISTRY_DATA}> variable in C<${REGISTRY_CONF}> file.
Comma separated list of tags to prepare. Available tags are "request",
"package" and "provides". The "request" refers to the whole thing and
is mandatory. The "package" includes name-version-release information
for all installed packages and "provides" adds more details required to
capture package options. For details see OPENPKG_PREP below.
Defaults to "request,package,provides"

=back

=head1 FILES

The following files are used by B<openpkg register>:

=over 4

=item OPENPKG_UUID=F<${PREFIX}/etc/openpkg/uuid>

 UUID_REGISTRY="..."
 UUID_INSTANCE="..."
 UUID_PLATFORM="..."

=item OPENPKG_CONF=F<${PREFIX}/etc/openpkg/register.conf>

Format suitable for shell evaluation. Interactive mode appends remarked date
and current settings for reuse as new defaults for future runs. Can be preset
to customize or automate registration.

=item OPENPKG_PREP=F<${PREFIX}/etc/openpkg/register.prep>

 <?xml version="1.0" encoding="iso-8859-1" standalone="no"?>
 <!DOCTYPE registry
   PUBLIC "-//OpenPKG//DTD OpenPKG Registry 0.0.1//EN"
   "http://registry.openpkg.org/registry.dtd" []>
 <registry>
     <request id="..."
         registry_user="me@example.com"
         registry_desc="openpkg://rm0.openpkg.net/openpkg-dev"
         registry_plat="ix86-freebsd6.1"
         registry_orel="OpenPKG-CURRENT"
         uuid_registry="..."
         uuid_instance="..."
         uuid_platform="..."
     />
     <package id="..." name="ssmtp" version="2.61" release="20050608">
        <provides name="ssmtp::with_ssl" flag="=" version="no"/>
        <provides name="MTA" flag="" version=""/>
        <provides name="ssmtp" flag="=" version="2.61-20050608"/>
     </package>
 </registry>

The XML request starts with <?xml version ...> and <!DOCTYPE registry ...>
headers followed by a <registry> container. The request is inside a <request>
tag. Successful submission into the XMLdump form requires the headers and
exactly one container. It is possible to merge multiple requests into a single
container manually or otherwise and submit them all at once.

=item OPENPKG_TRAN=F<${PREFIX}/etc/openpkg/register.tran>

 <?xml version="1.0" encoding="iso-8859-1" standalone="no"?>
 <!DOCTYPE registry
   PUBLIC "-//OpenPKG//DTD OpenPKG Registry 0.0.1//EN"
   "http://registry.openpkg.org/registry.dtd" []>
 <registry>
     <response id="..." done="yes">openpkg://rm0.openpkg.net/openpkg-dev</response>
 </registry>

The XML response starts with <?xml version ...> and <!DOCTYPE registry ...>
headers followed by a <registry> container. The response is inside a <request>
tag. The data carried in the tag comes from the "description" column on the
"association" page of the web form and can be edited on the server side.

=item OPENPKG_UTIL=F<${PREFIX}/etc/openpkg/register.util>

 REGISTRY_UUID="..."
 REGISTRY_DONE="yes"
 REGISTRY_RESP="openpkg://foo.example.com/my/openpkg/prefix"

=back

=head1 EXAMPLES

 # su - openpkg-mop
 $ openpkg register --printstatus
 REGISTRY_UUID="..."
 REGISTRY_DONE="yes"
 REGISTRY_RESP="openpkg://foo.example.com/my/openpkg/prefix"
 $ openpkg register --printstatus >/dev/null && echo "Yup"
 Yup
 $ eval `openpkg register --printstatus`; echo DONE=$REGISTRY_DONE
 DONE=yes
 $ openpkg register --mode=wipe
 $ openpkg register --printstatus || echo "Nope"
 Nope
 $ openpkg register --user=thl@openpkg.net --mode=post
 REGISTRY_UUID="...."
 REGISTRY_DONE="yes"
 REGISTRY_RESP="openpkg://foo.example.com/my/openpkg/prefix"
 $ openpkg register --printstatus >/dev/null && echo "Yup"
 Yup
 $ openpkg register --rewriteurls ftp://ftp.openpkg.org/foo/bar
 ftp://...:...@ftp.openpkg.org/foo/bar

=head1 SEE ALSO

bash(1), C<openpkg man uuid>, C<http://registry.openpkg.org/>

=head1 AUTHOR

Thomas Lotterer E<lt>thl@openpkg.orgE<gt>

=cut

